import os, re
from collections import OrderedDict

def readFasta_dict(InfileNameFN):
    Infile = open(InfileNameFN, "r")
    seqDictFN  = OrderedDict()
    for Line in Infile:
        Line = Line.rstrip("\n")
        if Line[0] == ">":
            Name            = Line
            seqDictFN[Name] = ""
        else:
            seqDictFN[Name] += Line
    Infile.close()
    return seqDictFN

def whiteSpaceAdd(recsFN1):
    longestNameLen = len(max(recsFN1.keys(), key = len))
    recsFN2        = OrderedDict()
    for name,value in recsFN1.items():
        nameWhiteSpace = name[1:] + " " * (longestNameLen - len(name) + 2)
        recsFN2[nameWhiteSpace] = value
    return recsFN2
    
def refAdd (refNumLineFN, seqDictFN3):
    recsFN4 = OrderedDict()
    recsFN4[">Num"] = "".join(refNumLineFN)
    for name, seq in seqDictFN3.items():
        recsFN4[name] = seq
    return recsFN4


def refNumLineMake (recsFN):
    seqLength = len(list(recsFN.values())[0])
    refNumLineFN = ["0"] * seqLength
    for name1, seq1 in recsFN.items():
        for name2, seq2 in recsFN.items():
            for i in range(seqLength):
                if seq1[i] == seq2[i] or \
                   seq1[i] == "X" or     \
                   seq2[i] == "X" or     \
                   seq1[i] == "N" or     \
                   seq2[i] == "N" or     \
                   seq1[i] == "-" or     \
                   seq2[i] == "-"      \
                   :
                   continue
                else:
                    refNumLineFN[i] = "1"
    return refNumLineFN


def matchFirster (keyNameFN, keySequenceFN, recsFN):
    recsFN1 = OrderedDict()
    for name, value in recsFN.items():
        sequence = ""
        if name == keyNameFN:
            for i in range(len(keySequenceFN)):
                sequence += value[i]
        else:
            for i in range(len(keySequenceFN)):
                if keySequenceFN[i] == "X" or keySequenceFN[i] == "-" or keySequenceFN[i] == "N":
                    sequence += value[i]
                elif keySequenceFN[i] == value[i]:
                    sequence += "."
                else:
                    sequence += value[i]
        recsFN1[name] = sequence
    return recsFN1


def keySpeciesPicker(keySpeciesFN, recsFN):
    r = re.compile(">" + keySpeciesFN)
    hitNames = [l for l in recsFN.keys() if r.match(l)]
    if len(hitNames) > 1:
        print("More than 2 keySpecies:%s. Stopped." % keySpeciesFN)
        exit()
    elif len(hitNames) == 0:
        print("No keySpecies:%s. Stopped." % keySpeciesFN)
        exit()
    else:
        return hitNames[0], recsFN[hitNames[0]]


## Main

InfileName  = "infile.txt"
keySpecies  = "S2"

recs       = readFasta_dict(InfileName)
refNumLine = refNumLineMake(recs)

keyName, keySequence = keySpeciesPicker(keySpecies, recs)
recs                 = matchFirster(keyName, keySequence, recs)

recs       = refAdd(refNumLine, recs)
recs       = whiteSpaceAdd(recs)


for name, seq in recs.items():
    print (name, seq)

#checkLine = list(recs.values())[0]
#for name, seq in recs.items():
#    print (name, end = "")
#    for i in range(len(checkLine)):
#        if checkLine[i] == "1":
#            print(seq[i], end="")
#    print()

